package main

import (
	"flag"
	"html/template"
	"log"
	"net/http"
	"os"

	"github.com/gorilla/websocket"
)

var addr = flag.String("addr", "0.0.0.0:15001", "http service address")

var upgrader = websocket.Upgrader{} // use defeult options

var file, _ = os.Create("file.log")
var logger = log.New(file, "", log.Ldate|log.Ltime)

func echo(w http.ResponseWriter, r *http.Request) {
	logger.Println("echo-RemoteAddr: ", r.RemoteAddr, "LocalAddr: ", r.Host+r.RequestURI)
	sign := req.URL.Query().Get("sign") // 一种在 连接建立前进行鉴权的方式，如果不通过，连接不会建立，比连接建立后再鉴权是更好的方式，一定程度上减少了很多不合法连接的建立，优化了资源
	log.Info("=================Sign:", sign)
	if sign != "udesk" {
		log.Error("Sign fail connect not build success")
		res.WriteHeader(403)
		return
	}

	conn, err := upgrader.Upgrade(w, r, nil) // 升级http为websocket

	if err != nil {
		log.Print("upgrade:", err)
		return
	}

	defer conn.Close()

	for { // 循环操作 读取客户端发送过来的数据
		mt, message, err := conn.ReadMessage()
		if err != nil {
			log.Println("read: ", err)
			break
		}

		log.Printf("recv: %s", message)      // 打印读取的数据
		err = conn.WriteMessage(mt, message) // 将收到的数据,又返回写给客户端
		if err != nil {
			log.Println("write:", err)
			break
		}
	}
}

func home(w http.ResponseWriter, r *http.Request) {
	logger.Println("home-RemoteAddr: ", r.RemoteAddr, "LocalAddr: ", r.RequestURI)
	homeTemplate.Execute(w, "ws://"+r.Host+"/echo")
}

func main() {
	flag.Parse()
	log.SetFlags(0)
	http.HandleFunc("/echo", echo)
	http.HandleFunc("/", home)
	log.Println("Server start...")
	log.Fatal(http.ListenAndServe(*addr, nil))
}

var homeTemplate = template.Must(template.New("").Parse(`
<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<script>
window.addEventListener("load", function(evt) {
    var output = document.getElementById("output");
    var input = document.getElementById("input");
    var ws;
    var print = function(message) {
        var d = document.createElement("div");
        d.innerHTML = message;
        output.appendChild(d);
    };
    document.getElementById("open").onclick = function(evt) {
        if (ws) {
            return false;
        }
        ws = new WebSocket("{{.}}");
        ws.onopen = function(evt) {
            print("OPEN");
        }
        ws.onclose = function(evt) {
            print("CLOSE");
            ws = null;
        }
        ws.onmessage = function(evt) {
            print("RESPONSE: " + evt.data);
        }
        ws.onerror = function(evt) {
            print("ERROR: " + evt.data);
        }
        return false;
    };
    document.getElementById("send").onclick = function(evt) {
        if (!ws) {
            return false;
        }
        print("SEND: " + input.value);
        ws.send(input.value);
        return false;
    };
    document.getElementById("close").onclick = function(evt) {
        if (!ws) {
            return false;
        }
        ws.close();
        return false;
    };
});
</script>
</head>
<body>
<table>
<tr><td valign="top" width="50%">
<p>Click "Open" to create a connection to the server,
"Send" to send a message to the server and "Close" to close the connection.
You can change the message and send multiple times.
<p>
<form>
<button id="open">Open</button>
<button id="close">Close</button>
<p><input id="input" type="text" value="Hello world!">
<button id="send">Send</button>
</form>
</td><td valign="top" width="50%">
<div id="output"></div>
</td></tr></table>
</body>
</html>
`))
